/*
	File:		ProcessTest.cp

	Contains:	TProcess is a Process Manager class.
  				ProcessTest.cp contains the testing code for testing TProcess.

	Written by: Kent Sandvik	

	Copyright:	Copyright  1992-1999 by Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):
				8/18/1999	Karl Groethe	Updated for Metrowerks Codewarror Pro 2.1
				

*/

// Include files
#ifndef _PROCESS_
#include "Process.h"
#endif


// This test program will first create a TProcess object that will probe the current process
// itself. The second test will iterate through processes in the list and print out information.
// The final test will find the Finder Process ('MACS').


void main(void)
{
	cout << "Start TProcess testing\n";

	// create default TProcess
	TProcess myProcess;

	cout << "First test, the current process\n";
	cout << "We have just now " << myProcess.GetNumProcesses() << " processes running.\n";
	cout << "This process has " << myProcess.GetFreeMem() << " memory free.\n";

	cout << "\nSecond test, iteration of processes\n";

	for (TProcess procs; !procs.Last(); procs.Next())
	{
		ProcessSerialNumber thePSN = procs.GetProcessID();
		cout << "High Process ID = " << thePSN.highLongOfPSN << "\n";
		cout << "Low Process ID = " << thePSN.lowLongOfPSN << "\n";

		cout << "Process size (bytes) = " << procs.GetProcessSize() << "\n";
		cout << "Free memory (bytes) = " << procs.GetFreeMem() << "\n";
		cout << "Launch date (seconds) = " << procs.GetLaunchDate() << "\n";
	}

	cout << "\nFinal test, try to find a process with signature 'MACS' (Finder)\n ";
	TProcess findProc;
	if (findProc.FindProcess('MACS'))
		cout << " Finder Process found!\n";

	cout << "\nEnd TProcess testing!\n";
}


// _________________________________________________________________________________________________________ //

/*	Change History (most recent last):
  No		Init.	Date		Comment
  1			khs		6/10/92		New file
  2			khs		7/6/92		First decent working class
*/
