/*
	File:		LateImportTest.c

	Contains:	CFM late import test framework.

	Written by:	Quinn

	Copyright:	Copyright  1999 by Apple Computer, Inc., all rights reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):

         <2>    18/10/99    Quinn   Expanded tests to cover some extra cases.
         <1>     15/6/99    Quinn   First checked in.
*/

/////////////////////////////////////////////////////////////////

// MoreIsBetter Setup

#include "MoreSetup.h"

// Mac OS Interfaces

#include <MacTypes.h>

// Standard C Interfaces

#include <stdio.h>

// MIB Interfaces

#include "CFMLateImport.h"

// Project Interfaces

#include "TestLibrary.h"

/////////////////////////////////////////////////////////////////
#pragma mark ----- Test Framework -----

static void CallLateImportedRoutines(void)
	// This routine uses the data and code imports we just
	// late imported.  I put it in a separate routine because
	// the compiler was caching certain variables (such as
	// the address of gTestData) in registers and wouldn't let go.
	// Putting these accesses in a separate procedure defeats that
	// optimisation and allows the code to work as expected.
	// Something to be wary of.
{
	MoreAssertQ(&gTestData != nil);
	MoreAssertQ(TestRoutine1 != nil);
	MoreAssertQ(TestRoutine2 != nil);

	gTestData = 0;
	TestRoutine1();
	TestRoutine2();
	MoreAssertQ(gTestData == 3);
	printf("gTestData = %ld\n", gTestData);
}

static CFragConnectionID 			gMyCFragConnection = nil;
static CFragSystem7DiskFlatLocator	gMyFragLocator;
static FSSpec						gMyFragFSSpec;

void main(void)
{
	OSStatus err;
	CFragConnectionID implConn;
	Ptr junkMain;
	Str255 junkMessage;
	FSSpec fss;
	
	printf("Hello Cruel World!\n");
	printf("LateImportTest.c\n");
	
	MoreAssertQ(gMyCFragConnection != nil);

	err = noErr;
	if (TestRoutine1 == nil || TestRoutine2 == nil) {
		err = FSMakeFSSpec(0, 0, "\p:TestLibrary:TestLibraryImpl", &fss);
		if (err == noErr) {
			err = GetDiskFragment(&fss, 0, kCFragGoesToEOF, fss.name, kReferenceCFrag, &implConn, &junkMain, junkMessage);
		}
		if (err == noErr) {
			err = CFMLateImportLibrary(&gMyFragLocator, gMyCFragConnection, "\p__LateImportDummyCode", "\pTestLibrary", implConn);
		}
	}
	if (err == noErr) {
		CallLateImportedRoutines();
	}
	
	if (err == noErr) {
		printf("Success!\n");
	} else {
		printf("Failed with error %ld.\n", err);
	}
	
	printf("Done.  Press command-Q to Quit.\n");
}

extern void __LateImportDummyCode(void);

#pragma export list __LateImportDummyCode

extern void __LateImportDummyCode(void)
{
	// No need to have anything here because a) it's never
	// called, and b) the fact that it's exported prevents
	// the linker from dead stripping it.
}

extern OSErr MyCFragInitFunction(const CFragInitBlock *initBlock);

extern OSErr MyCFragInitFunction(const CFragInitBlock *initBlock)
{
	gMyCFragConnection = initBlock->connectionID;
	MoreAssertQ(initBlock->fragLocator.where == kDataForkCFragLocator);
	gMyFragFSSpec  = *initBlock->fragLocator.u.onDisk.fileSpec;
	gMyFragLocator = initBlock->fragLocator.u.onDisk;
	gMyFragLocator.fileSpec = &gMyFragFSSpec;
	return noErr;
}
