(*
	File:		TradDriverLoaderLib.p

	Contains:	Pascal interface for the pseudo-DriverLoaderLib for 'DRVR's.

	Written by:	Quinn

	Copyright:	 1996-1999 by Apple Computer, Inc., all rights reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):

         <1>     25/2/99    Quinn   First checked in.
*)

unit TradDriverLoaderLib;

interface

	uses
		
		// MoreIsBetter Setup

		MoreSetup,

		// Mac OS Interfaces

		Types,
		Devices;

	(* The following types are not in the "Devices.p".  I need them 
		because TradGetDriverInformation uses pointers instead
		of var parameters to make it easy to pass nil to
		the parameters you're not interested in.
	*)

	type
		DriverFlagsPtr = ^DriverFlags;
		UnitNumberPtr = ^UnitNumber;
		DriverRefNumPtr = ^DriverRefNum;
		DRVRHeaderPtrPtr = ^DRVRHeaderPtr;
	
	(* See "TradDriverLoaderLib.h" for comments.  I couldn't be bothered
		replicating them in both files.
	*)

	function TradHigherDriverVersion(var driverVersion1 : NumVersion;
										var driverVersion2 : NumVersion) : Boolean;
	function TradHighestUnitNumber : UnitNumber;
	function TradDriverGestaltOn(refNum : DriverRefNum) : OSErr;
	function TradDriverGestaltOff(refNum : DriverRefNum) : OSErr;
	function TradDriverGestaltIsOn(flags : DriverFlags) : Boolean;
	function TradLookupDrivers(beginningUnit : UnitNumber;
								endingUnit : UnitNumber;
								emptyUnits : Boolean;
								var returnedRefNums : ItemCount; 
								refNums : DriverRefNumPtr) : OSErr;
	function TradInstallDriverFromPtr(driver : DRVRHeaderPtr;
										beginningUnit : UnitNumber;
										endingUnit : UnitNumber;
										var refNum : DriverRefNum) : OSErr;
	function TradInstallDriverFromHandle(driver : DRVRHeaderHandle;
											beginningUnit : UnitNumber;
											endingUnit : UnitNumber;
											var refNum : DriverRefNum) : OSErr;
	function TradInstallDriverFromResource(rsrcID : SInt16; rsrcName : StringPtr;
											beginningUnit : UnitNumber;
											endingUnit : UnitNumber;
											var refNum : DriverRefNum) : OSErr;
	function TradGetDriverInformation(refNum : DriverRefNum;
										thisUnit : UnitNumberPtr;
										flags : DriverFlagsPtr;
										name : StringPtr;
										driverHeader : DRVRHeaderPtrPtr
										) : OSErr;
	function TradOpenInstalledDriver(refNum : DriverRefNum; ioPermission : SInt8) : OSErr;
	function TradRemoveDriver(refNum : DriverRefNum; immediate : Boolean) : OSErr;
	function TradRenameDriver(refNum : DriverRefNum; newDriverName : Str255) : OSErr;

end. (* TradDriverLoaderLib *)

