/*
	File:		TestMoreSystemMenus.c

	Contains:	System extension to test the MoreSystemMenus module.

	Written by:	Quinn

	Copyright:	Copyright  1999 by Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):

         <1>     27/8/99    Quinn   First checked in.
*/

/////////////////////////////////////////////////////////////////

// MIB Setup

#include "MoreSetup.h"

// Mac OS Interfaces

#include <Resources.h>
#include <Menus.h>
#include <TextUtils.h>
#include <Sound.h>
#include <Dialogs.h>

// Wacky Metrowerks Interfaces

#include <A4Stuff.h>
#include <SetUpA4.h>

// MIB Interfaces

#include "MoreSystemMenus.h"

/////////////////////////////////////////////////////////////////

static FSSpec gOurExtensionFile;

static void DoModalDialog(ConstStr255Param str)
{
	OSStatus err;
	SInt16 refNum;
	DialogPtr dlg;
	DialogItemType junkType;
	Handle itemH;
	Rect junkRect;
	DialogItemIndex itemHit;
	
	// When opening resource files in a weird context, use
	// fsRdPerm avoid being bitten by an annoying Resource Manager
	// misfeature.  See TN 1120 for details.
	//
	// <http://developer.apple.com/technotes/tn/tn1120.html>
	
	refNum = FSpOpenResFile(&gOurExtensionFile, fsRdPerm);
	err = ResError();
	if (err == noErr) {
		dlg = GetNewDialog(128, nil, (WindowPtr) -1);
		if (dlg == nil) {
			err = memFullErr;
		}
		
		if (err == noErr) {
			GetDialogItem(dlg, 3, &junkType, &itemH, &junkRect);
			SetDialogItemText(itemH, str);
			SetDialogDefaultItem(dlg, 1);
			ShowWindow(dlg);
			do {
				ModalDialog(nil, &itemHit);
			} while (itemHit != 1);
			
			DisposeDialog(dlg);
		}
		
		CloseResFile(refNum);
		MoreAssertQ(ResError() == noErr);
	}
	
	if (err != noErr) {
		DebugStr("\pDoModalDialog: Got an error.");
	}
}

static MenuRef gSysMenu = nil;
static MenuRef gSubMenu1 = nil;
static MenuRef gSubMenu2 = nil;

static pascal OSStatus MySysMenuCallback(OSType selector,
											MenuRef theMenu,
											void *param, 
											void *refcon)
{
	long oldA4;
	OSStatus err;
	Str255 itemStr;
	UInt16 hitItem;
	EventRecord event;
	#pragma unused(refcon)
	
	oldA4 = SetUpA4();

	switch (selector) {
		case kSystemMenuActOnChosen:
			hitItem = (UInt16) param;
			GetMenuItemText(theMenu, hitItem, itemStr);
			if (hitItem == 1) {
				DoModalDialog(itemStr);
			} else {
				DebugStr(itemStr);
			}
			break;
		case kSystemMenuPreSelectAdjust:
			err = InsertSystemSubMenu(gSysMenu, gSysMenu, 4, gSubMenu1);
			if (err == noErr) {
				err = InsertSystemSubMenu(gSysMenu, gSubMenu1, 2, gSubMenu2);
			}
			if (err == noErr) {
				(void) OSEventAvail(0, &event);
				if (event.modifiers & optionKey) {
					DoModalDialog("\pkSystemMenuPreSelectAdjust");
				}
			}
			MoreAssertQ(err == noErr);
			break;
		default:
			// do nothing
			break;
	}

	(void) SetA4(oldA4);
	return noErr;
}

static OSStatus BasicSystemMenuTest(void)
{
	OSStatus err;
	OSStatus junk;
	IconSuiteRef iconSuite;
	
	// Initialize our system menu.
	
	iconSuite = nil;
	gSysMenu = nil;
	
	gSysMenu  = GetNewSystemMenu(128);
	gSubMenu1 = GetNewSystemMenu(129);
	gSubMenu2 = GetNewSystemMenu(130);
	if (gSysMenu == nil || gSubMenu1 == nil || gSubMenu2 == nil) {
		err = memFullErr;
	}
	if (err == noErr) {
		err = GetDetachedIconSuite(&iconSuite, 128, kSelectorAllSmallData);
	}
	if (err == noErr) {
		err = SetMenuTitleToIconSuite(gSysMenu, iconSuite);
	}
	if (err == noErr) {
		err = InsertSystemMenu(gSysMenu, MySysMenuCallback, nil);
	}
	
	// Clean up.

	if (err != noErr) {
		if (gSysMenu != nil) {
			DisposeMenu(gSysMenu);
			gSysMenu = nil;
		}
		if (gSubMenu1 != nil) {
			DisposeMenu(gSubMenu1);
			gSubMenu1 = nil;
		}
		if (gSubMenu2 != nil) {
			DisposeMenu(gSubMenu2);
			gSubMenu2 = nil;
		}
		if (iconSuite != nil) {
			junk = DisposeIconSuite(iconSuite, true);
			MoreAssertQ(junk == noErr);
		}
	}
	
	return err;
}

/////////////////////////////////////////////////////////////////

static MenuRef gSoundMenu = nil;

static pascal OSStatus SoundSysMenuCallback(OSType selector,
											MenuRef theMenu,
											void *param, 
											void *refcon)
{
	long oldA4;
	OSStatus junk;
	SndListHandle sndH;
	Str255 itemStr;
	#pragma unused(param)
	#pragma unused(refcon)
	
	oldA4 = SetUpA4();

	switch (selector) {
		case kSystemMenuActOnChosen:
			MoreAssertQ(theMenu == gSoundMenu);
			
			GetMenuItemText(theMenu, (UInt16) param, itemStr);
			sndH = (SndListHandle) GetNamedResource('snd ', itemStr);
			if (sndH != nil) {
				junk = SndPlay(nil, sndH, false);
				MoreAssert(junk == noErr);
			} else {
				MoreAssert(false);
			}
			break;
		default:
			// do nothing
			break;
	}

	(void) SetA4(oldA4);
	return noErr;
}

static OSStatus CreateSoundSystemMenu(void)
{
	OSStatus err;
	OSStatus junk;
	IconSuiteRef soundIcon;
	Str255 soundMenuTitle;
	
	err = GetDetachedIconSuite(&soundIcon, 256, kSelectorAllSmallData);
	if (err == noErr) {
		IconSuiteToSystemMenuTitle(soundIcon, soundMenuTitle);

		gSoundMenu = NewSystemMenu(soundMenuTitle);
		if (gSoundMenu == nil) {
			err = memFullErr;
		}
	}
	if (err == noErr) {
		AppendResMenu(gSoundMenu, 'snd ');
		err = InsertSystemMenu(gSoundMenu, SoundSysMenuCallback, nil);
	}
	
	// Clean up.
	
	if (err != noErr) {
		if (gSoundMenu != nil) {
			DisposeMenu(gSoundMenu);
			gSoundMenu = nil;
		}
		if (soundIcon != nil) {
			junk = DisposeIconSuite(soundIcon, true);
			MoreAssertQ(junk == noErr);
		}
	}
	
	return err;
}


/////////////////////////////////////////////////////////////////

static OSErr RefNumToFSSpec(SInt16 refNum, FSSpec *fss)
{
	OSErr err;
	FCBPBRec fcbPB;
	
    fcbPB.ioNamePtr = fss->name;
    fcbPB.ioVRefNum = 0;
    fcbPB.ioRefNum = refNum;
    fcbPB.ioFCBIndx = 0;
    err = PBGetFCBInfoSync(&fcbPB);
	if (err == noErr) {
		fss->vRefNum = fcbPB.ioFCBVRefNum;
		fss->parID = fcbPB.ioFCBParID;
	}
	return err;
}

extern void main(void)
{
	long oldA4;
	OSStatus err;
	Handle ourCodeResource;
	Boolean shouldDetach;
	
	#if MORE_DEBUG
		Debugger();
	#endif
	
	oldA4 = SetCurrentA4();
	RememberA4();
	
	// Find, lock and detach our code.
	
	ourCodeResource = Get1Resource('INIT', 128);
	MoreAssertQ(ourCodeResource != nil);

	HLock(ourCodeResource);
	MoreAssertQ(MemError() == noErr);
	
	shouldDetach = false;
	
	err = RefNumToFSSpec(CurResFile(), &gOurExtensionFile);
	if (err == noErr) {
		err = InitMoreSystemMenus();
		
		// Once InitMoreSystemMenus returns noErr, we're patched into
		// the system in such a way that we can't unload our code.
		
		shouldDetach = (err == noErr);
	}
	if (err == noErr) {
		err = BasicSystemMenuTest();
	}
	if (err == noErr) {
		err = CreateSoundSystemMenu();
	}
	if (err != noErr) {
		DebugStr("\pTestMoreSystemMenus: Failed to start up");
	}

	if (shouldDetach) {
		DetachResource(ourCodeResource);
		MoreAssertQ(ResError() == noErr);
	}
	
	(void) SetA4(oldA4);
}