/*
	File:		MoreProcesses.cp

	Contains:	

	Written by:	Pete Gontier

	Copyright:	Copyright (c) 1998 Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):

         <3>     1/22/99    PCG     TARGET_CARBON
         <2>    11/11/98    PCG     fix header
         <1>    11/10/98    PCG     first big re-org at behest of Quinn

	Old Change History (most recent first):

         <3>     11/9/98    PCG     fix comment 2
         <2>     11/9/98    PCG     add copyright blurb
         <1>     6/16/98    PCG     initial checkin
*/

#include "MoreProcesses.h"

pascal OSStatus GetSomeProcessInfo (const ProcessSerialNumber *psn, ProcessInfoRec *pir)
{
	if (!MoreAssert (pir))
		return paramErr;

	pir->processInfoLength	= sizeof (*pir);
	pir->processName		= nil;
	pir->processAppSpec		= nil;

	if (psn)
		return GetProcessInformation (psn,pir);
	else
	{
		ProcessSerialNumber psn = { kNoProcess, kCurrentProcess };
		return GetProcessInformation (&psn,pir);
	}
}

pascal OSStatus IsProcessBackgroundOnly (const ProcessSerialNumber *psn, Boolean *isBOA)
{
	OSStatus err = noErr;

	ProcessInfoRec pir;

	if (!(err = GetSomeProcessInfo (psn,&pir)))
	{
		*isBOA = (modeOnlyBackground & pir.processMode) ? true : false;
	}

	return err;
}

pascal OSStatus FindProcessBySignature (OSType processSignature, ProcessSerialNumber *psn)
{
	OSStatus err = noErr;

	if (!(MoreAssert (psn)))
		err = paramErr;
	else
	{
		psn->lowLongOfPSN	= kNoProcess;
		psn->highLongOfPSN	= kNoProcess;

		while (!(err = GetNextProcess (psn)))
		{
			ProcessInfoRec pir;

			if (!(err = GetSomeProcessInfo (psn,&pir)))
				if (processSignature == pir.processSignature)
					break;
		}
	}

	return err;
}
