/*
	File:		MoreSound.cp

	Contains:	helper functions for simple use of Sound Manager

	Written by:	Pete Gontier

	Copyright:	Copyright  1998, Apple Computer, Inc. All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):

         <2>    11/11/98    PCG     fix header
         <1>    11/10/98    PCG     first big re-org at behest of Quinn

	Old Change History (most recent first):

         <1>    10/23/98    PCG     initial check-in
*/

#include "MoreSound.h"

pascal OSErr NewDigitizedSoundChannel (UInt16 qLength, SndChannelPtr *result)
{
	OSErr err = noErr;

	*result = (SndChannelPtr) ::NewPtr ((sizeof (**result) - sizeof ((**result).queue)) + (sizeof ((**result).queue[0]) * qLength));

	if (!*result)
		err = ::MemError ( );
	else
	{
		(**result).qLength = qLength;
		err = ::SndNewChannel (result,0,0,nil);
		if (err) ::DisposePtr (Ptr (*result));
	}

	return err;
}

pascal OSErr DisposeSoundChannel (SndChannelPtr chan)
{
	OSErr err = ::SndDisposeChannel (chan,true);

	if (!err)
	{
		::DisposePtr (Ptr (chan));
		MoreAssert (MemError ( ) == noErr);
	}

	return err;
}

pascal OSErr PlaySoundListHandle (SndChannelPtr chan, SndListHandle hand, Boolean asynch)
{
	OSErr err = noErr;

	if (!MoreAssert (chan && hand && *hand))
		err = paramErr;
	else
	{

#if MORE_DEBUG

		if (asynch)
		{
			SInt8 hState = ::HGetState (Handle (hand));
			if (MoreAssert (!(err = MemError ( ))))
			{
				(void) MoreAssert (hState & 0x80); // is it locked?
			}
		}

#endif

		err = ::SndPlay (chan,hand,asynch);
	}

	return err;
}

pascal OSErr IsSoundChannelPlaying (SndChannelPtr chan, Boolean *isPlaying)
{
	OSErr err = noErr;

	if (!MoreAssert (chan))
		err = paramErr;
	else
	{
		SCStatus status;

		if (!(err = ::SndChannelStatus (chan,sizeof(status),&status)))
			*isPlaying = status.scChannelBusy && !(status.scChannelPaused);
	}

	return err;
}
